@@ -17,16 +17,18 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 """
     usage:
-           mock [options] {--init|--clean}
+           mock [options] {--init|--clean|--scrub=[all,chroot,cache,root-cache,c-cache,yum-cache]}
            mock [options] [--rebuild] /path/to/srpm(s)
+           mock [options] --buildsrpm {--spec /path/to/spec --sources /path/to/src|--scm-enable [--scm-option key=value]}
            mock [options] {--shell|--chroot} <cmd>
            mock [options] --installdeps {SRPM|RPM}
            mock [options] --install PACKAGE
            mock [options] --copyin path [..path] destination
            mock [options] --copyout path [..path] destination
+           mock [options] --scm-enable [--scm-option key=value]
 """
 
 # library imports
@@ -40,17 +42,17 @@ import pwd
 import sys
 import time
 from optparse import OptionParser
-from socket import gethostname
+from glob import glob
 
 # all of the variables below are substituted by the build system
 __VERSION__ = "unreleased_version"
 SYSCONFDIR = os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), "..", "etc")
 PYTHONDIR = os.path.dirname(os.path.realpath(sys.argv[0]))
-PKGPYTHONDIR = os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), "mock")
+PKGPYTHONDIR = os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), "mockbuild")
 MOCKCONFDIR = os.path.join(SYSCONFDIR, "mock")
 # end build system subs
 
-# import all mock.* modules after this.
+# import all mockbuild.* modules after this.
 sys.path.insert(0, PYTHONDIR)
 
 # set up basic logging until config file can be read
@@ -59,11 +61,15 @@ logging.basicConfig(format=FORMAT, level=logging.WARNING)
 log = logging.getLogger()
 
 # our imports
-import mock.exception
-from mock.trace_decorator import traceLog, decorate
-import mock.backend
-import mock.uid
-import mock.util
+import mockbuild.exception
+from mockbuild.trace_decorator import traceLog, decorate
+import mockbuild.backend
+import mockbuild.uid
+import mockbuild.util
+
+def scrub_callback(option, opt, value, parser):
+    parser.values.scrub.append(value)
+    parser.values.mode = "clean"
 
 def command_parse(config_opts):
     """return options and args from parsing the command line"""
@@ -75,7 +81,7 @@ def command_parse(config_opts):
                       help="rebuild the specified SRPM(s)")
     parser.add_option("--buildsrpm", action="store_const", const="buildsrpm",
                       dest="mode",
-                      help="Build a SRPM from spec (--spec ...) and sources (--sources ...)")
+                      help="Build a SRPM from spec (--spec ...) and sources (--sources ...) or from SCM")
     parser.add_option("--shell", action="store_const",
                       const="shell", dest="mode",
                       help="run the specified command interactively within the chroot."
@@ -86,6 +92,12 @@ def command_parse(config_opts):
     parser.add_option("--clean", action="store_const", const="clean",
                       dest="mode",
                       help="completely remove the specified chroot")
+    scrub_choices = ('chroot', 'cache', 'root-cache', 'c-cache', 'yum-cache', 'all')
+    scrub_metavar = "[all|chroot|cache|root-cache|c-cache|yum-cache]"
+    parser.add_option("--scrub", action="callback", type="choice", default=[],
+                      choices=scrub_choices, metavar=scrub_metavar,
+                      callback=scrub_callback,
+                      help="completely remove the specified chroot or cache dir or all of the chroot and cache")
     parser.add_option("--init", action="store_const", const="init", dest="mode",
                       help="initialize the chroot, do not build anything")
     parser.add_option("--installdeps", action="store_const", const="installdeps",
@@ -97,6 +109,9 @@ def command_parse(config_opts):
     parser.add_option("--update", action="store_const", const="update",
                       dest="mode",
                       help="update installed packages using yum")
+    parser.add_option("--remove", action="store_const", const="remove",
+                      dest="mode",
+                      help="remove packages using yum")
     parser.add_option("--orphanskill", action="store_const", const="orphanskill",
                       dest="mode",
                       help="Kill all processes using specified buildroot.")
@@ -128,6 +143,10 @@ def command_parse(config_opts):
                       dest="cleanup_after", default=None,
                       help="Dont clean chroot after building. If automatic"
                            " cleanup is enabled, use this to disable.", )
+    parser.add_option("--cache-alterations", action="store_true",
+                      dest="cache_alterations", default=False,
+                      help="Rebuild the root cache after making alterations to the chroot"
+                           " (i.e. --install). Only useful when using tmpfs plugin.")
     parser.add_option("--arch", action ="store", dest="arch",
                       default=None, help="Sets kernel personality().")
     parser.add_option("--target", action ="store", dest="rpmbuild_arch",
@@ -162,9 +181,10 @@ def command_parse(config_opts):
                            " before running command when using --chroot")
 
     parser.add_option("--spec", action="store",
-                    help="Specifies spec file to use to build an SRPM (used only with --buildsrpm)")
+                      help="Specifies spec file to use to build an SRPM (used only with --buildsrpm)")
     parser.add_option("--sources", action="store",
-                    help="Specifies sources to use to build an SRPM (used only with --buildsrpm)")
+                      help="Specifies sources (either a single file or a directory of files)"
+                      "to use to build an SRPM (used only with --buildsrpm)")
 
     # verbosity
     parser.add_option("-v", "--verbose", action="store_const", const=2,
@@ -183,172 +203,56 @@ def command_parse(config_opts):
                       dest="disabled_plugins", type="string", default=[],
                       help="Disable plugin. Currently-available plugins: %s"
                            % repr(config_opts['plugins']))
+    parser.add_option("--plugin-option", action="append", dest="plugin_opts",
+                      default=[], type="string",
+                      metavar="PLUGIN:KEY=VALUE",
+                      help="define an plugin option (may be used more than once)")
 
     parser.add_option("--print-root-path", help="print path to chroot root",
                       dest="printrootpath", action="store_true",
                       default=False)
 
+    # SCM options
+    parser.add_option("--scm-enable", help="build from SCM repository",
+                      dest="scm", action="store_true",
+                      default=None)
+    parser.add_option("--scm-option", action="append", dest="scm_opts",
+                      default=[], type="string",
+                      help="define an SCM option (may be used more than once)")
+
     (options, args) = parser.parse_args()
+
+    # handle old-style commands
     if len(args) and args[0] in ('chroot', 'shell',
-            'rebuild', 'install', 'installdeps', 'init', 'clean'):
+            'rebuild', 'install', 'installdeps', 'remove', 'init', 'clean'):
         options.mode = args[0]
         args = args[1:]
 
+    # explicitly disallow multiple targets in --target argument
+    if options.rpmbuild_arch:
+        if options.rpmbuild_arch.find(',') != -1:
+                   raise mockbuild.exception.BadCmdline, "--target option accepts only one arch. Invalid: %s" % options.rpmbuild_arch
+
+    if options.mode == 'buildsrpm' and not (options.spec and options.sources):
+        if not options.scm:
+            raise mockbuild.exception.BadCmdline, "Must specify both --spec and --sources with --buildsrpm"
+    if options.spec:
+        options.spec = os.path.expanduser(options.spec)
+    if options.sources:
+        options.sources = os.path.expanduser(options.sources)
+
     return (options, args)
 
 decorate(traceLog())
-def setup_default_config_opts(config_opts, unprivUid):
-    "sets up default configuration."
-    # global
-    config_opts['version'] = __VERSION__
-    config_opts['basedir'] = '/var/lib/mock' # root name is automatically added to this
-    config_opts['resultdir'] = '%(basedir)s/%(root)s/result'
-    config_opts['cache_topdir'] = '/var/cache/mock'
-    config_opts['clean'] = True
-    config_opts['chroothome'] = '/builddir'
-    config_opts['log_config_file'] = 'logging.ini'
-    config_opts['rpmbuild_timeout'] = 0
-    config_opts['chrootuid'] = unprivUid
+def check_arch_combination(target_arch, config_opts):
     try:
-        config_opts['chrootgid'] = grp.getgrnam("mock")[2]
+        legal = config_opts['legal_host_arches']
     except KeyError:
-        #  'mock' group doesnt exist, must set in config file
-        pass
-    config_opts['build_log_fmt_name'] = "unadorned"
-    config_opts['root_log_fmt_name']  = "detailed"
-    config_opts['state_log_fmt_name'] = "state"
-    config_opts['online'] = True
-
-    config_opts['internal_dev_setup'] = True
-    config_opts['internal_setarch'] = True
-
-    # cleanup_on_* only take effect for separate --resultdir
-    # config_opts provides fine-grained control. cmdline only has big hammer
-    config_opts['cleanup_on_success'] = 1
-    config_opts['cleanup_on_failure'] = 1
-
-    # (global) plugins and plugin configs.
-    # ordering constraings: tmpfs must be first.
-    #    root_cache next.
-    #    after that, any plugins that must create dirs (yum_cache)
-    #    any plugins without preinit hooks should be last.
-    config_opts['plugins'] = ('tmpfs', 'root_cache', 'yum_cache', 'bind_mount', 'ccache')
-    config_opts['plugin_dir'] = os.path.join(PKGPYTHONDIR, "plugins")
-    config_opts['plugin_conf'] = {
-            'ccache_enable': True,
-            'ccache_opts': {
-                'max_cache_size': "4G",
-                'dir': "%(cache_topdir)s/%(root)s/ccache/"},
-            'yum_cache_enable': True,
-            'yum_cache_opts': {
-                'max_age_days': 30,
-                'max_metadata_age_days': 30,
-                'dir': "%(cache_topdir)s/%(root)s/yum_cache/",
-                'online': True,},
-            'root_cache_enable': True,
-            'root_cache_opts': {
-                'max_age_days': 15,
-                'dir': "%(cache_topdir)s/%(root)s/root_cache/",
-                'compress_program': 'gzip',
-                'extension': '.gz'},
-            'bind_mount_enable': True,
-            'bind_mount_opts': {'dirs': [
-                # specify like this:
-                # ('/host/path', '/bind/mount/path/in/chroot/' ),
-                # ('/another/host/path', '/another/bind/mount/path/in/chroot/'),
-                ]},
-            'tmpfs_enable': False,
-            'tmpfs_opts': {'required_ram_mb': 900},
-            }
-
-    # dependent on guest OS
-    config_opts['useradd'] = \
-        '/usr/sbin/useradd -o -m -u %(uid)s -g %(gid)s -d %(home)s -n %(user)s'
-    config_opts['use_host_resolv'] = True
-    config_opts['chroot_setup_cmd'] = 'groupinstall buildsys-build'
-    config_opts['target_arch'] = 'i386'
-    config_opts['rpmbuild_arch'] = None # <-- None means set automatically from target_arch
-    config_opts['yum.conf'] = ''
-    config_opts['more_buildreqs'] = {}
-    config_opts['files'] = {}
-    config_opts['macros'] = {
-        '%_topdir': '%s/build' % config_opts['chroothome'],
-        '%_rpmfilename': '%%{NAME}-%%{VERSION}-%%{RELEASE}.%%{ARCH}.rpm',
-        }
-
-decorate(traceLog())
-def set_config_opts_per_cmdline(config_opts, options, args):
-    "takes processed cmdline args and sets config options."
-    # do some other options and stuff
-    if options.arch:
-        config_opts['target_arch'] = options.arch
-    if options.rpmbuild_arch:
-        config_opts['rpmbuild_arch'] = options.rpmbuild_arch
-    elif config_opts['rpmbuild_arch'] is None:
-        config_opts['rpmbuild_arch'] = config_opts['target_arch']
-
-    if not options.clean:
-        config_opts['clean'] = options.clean
-
-    for option in options.rpmwith:
-        options.rpmmacros.append("_with_%s 1" % option)
-
-    for option in options.rpmwithout:
-        options.rpmmacros.append("_without_%s 1" % option)
-
-    for macro in options.rpmmacros:
-        try:
-            k, v = macro.split(" ", 1)
-            if not k.startswith('%'):
-                k = '%%%s' % k
-            config_opts['macros'].update({k: v})
-        except:
-            raise mock.exception.BadCmdline(
-                "Bad option for '--define' (%s).  Use --define 'macro expr'"
-                % macro)
-
-    if options.resultdir:
-        config_opts['resultdir'] = os.path.expanduser(options.resultdir)
-    if options.uniqueext:
-        config_opts['unique-ext'] = options.uniqueext
-    if options.rpmbuild_timeout is not None:
-        config_opts['rpmbuild_timeout'] = options.rpmbuild_timeout
-
-    for i in options.disabled_plugins:
-        if i not in config_opts['plugins']:
-            raise mock.exception.BadCmdline(
-                "Bad option for '--disable-plugins=%s'. Expecting one of: %s"
-                % (i, config_opts['plugins']))
-        config_opts['plugin_conf']['%s_enable' % i] = False
-    for i in options.enabled_plugins:
-        if i not in config_opts['plugins']:
-            raise mock.exception.BadCmdline(
-                "Bad option for '--enable-plugins=%s'. Expecting one of: %s"
-                % (i, config_opts['plugins']))
-        config_opts['plugin_conf']['%s_enable' % i] = True
-
-    if options.cleanup_after and not options.resultdir:
-        raise mock.exception.BadCmdline(
-            "Must specify --resultdir when using --cleanup-after")
-
-    if options.mode in ("rebuild",) and len(args) > 1 and not options.resultdir:
-        raise mock.exception.BadCmdline(
-            "Must specify --resultdir when building multiple RPMS.")
-
-    if options.cleanup_after == False:
-        config_opts['cleanup_on_success'] = False
-        config_opts['cleanup_on_failure'] = False
-
-    if options.cleanup_after == True:
-        config_opts['cleanup_on_success'] = True
-        config_opts['cleanup_on_failure'] = True
-
-    # cant cleanup unless separate resultdir
-    if not options.resultdir:
-        config_opts['cleanup_on_success'] = False
-        config_opts['cleanup_on_failure'] = False
-
-    config_opts['online'] = options.online
+        return
+    host_arch = os.uname()[-1]
+    if host_arch not in legal:
+        raise mockbuild.exception.InvalidArchitecture(
+            "Cannot build target %s on arch %s" % (target_arch, host_arch))
 
 decorate(traceLog())
 def do_rebuild(config_opts, chroot, srpms):
@@ -358,7 +262,7 @@ def do_rebuild(config_opts, chroot, srpms):
         sys.exit(50)
 
     # check that everything is kosher. Raises exception on error
-    for hdr in mock.util.yieldSrpmHeaders(srpms):
+    for hdr in mockbuild.util.yieldSrpmHeaders(srpms):
         pass
 
     start = time.time()
@@ -366,7 +270,8 @@ def do_rebuild(config_opts, chroot, srpms):
         for srpm in srpms:
             start = time.time()
             log.info("Start(%s)  Config(%s)" % (srpm, chroot.sharedRootName))
-            if config_opts['clean'] and chroot.state() != "clean":
+            if config_opts['clean'] and chroot.state() != "clean" \
+                    and not config_opts['scm']:
                 chroot.clean()
             chroot.init()
             chroot.build(srpm, timeout=config_opts['rpmbuild_timeout'])
@@ -378,6 +283,15 @@ def do_rebuild(config_opts, chroot, srpms):
         if config_opts["cleanup_on_success"]:
             log.info("Cleaning up build root ('clean_on_success=True')")
             chroot.clean()
+
+        if config_opts["createrepo_on_rpms"]:
+            log.info("Running createrepo on binary rpms in resultdir")
+            chroot.uidManager.dropPrivsTemp()
+            cmd = config_opts["createrepo_command"].split()
+            cmd.append(chroot.resultdir)
+            mockbuild.util.do(cmd)
+            chroot.uidManager.restorePrivs()
+
     except (Exception, KeyboardInterrupt):
         elapsed = time.time() - start
         log.error("Exception(%s) Config(%s) %d minutes %d seconds"
@@ -389,18 +303,21 @@ def do_rebuild(config_opts, chroot, srpms):
         raise
 
 def do_buildsrpm(config_opts, chroot, options, args):
+    # verify the input command line arguments actually exist
+    if not os.path.isfile(options.spec):
+        raise mockbuild.exception.BadCmdline, \
+            "input specfile does not exist: %s" % options.spec
+    if not os.path.isdir(options.sources):
+        raise mockbuild.exception.BadCmdline, \
+            "input sources directory does not exist: %s" % options.sources
     start = time.time()
     try:
-        # TODO: validate spec path (exists)
-        # TODO: validate SOURCES path (exists)
-
         log.info("Start(%s)  Config(%s)" % (os.path.basename(options.spec), chroot.sharedRootName))
         if config_opts['clean'] and chroot.state() != "clean":
             chroot.clean()
         chroot.init()
 
-        chroot.buildsrpm(spec=options.spec, sources=options.sources, timeout=config_opts['rpmbuild_timeout'])
-
+        srpm = chroot.buildsrpm(spec=options.spec, sources=options.sources, timeout=config_opts['rpmbuild_timeout'])
         elapsed = time.time() - start
         log.info("Done(%s) Config(%s) %d minutes %d seconds"
             % (os.path.basename(options.spec), config_opts['chroot_name'], elapsed//60, elapsed%60))
@@ -410,6 +327,8 @@ def do_buildsrpm(config_opts, chroot, options, args):
             log.info("Cleaning up build root ('clean_on_success=True')")
             chroot.clean()
 
+        return srpm
+
     except (Exception, KeyboardInterrupt):
         elapsed = time.time() - start
         log.error("Exception(%s) Config(%s) %d minutes %d seconds"
@@ -420,10 +339,37 @@ def do_buildsrpm(config_opts, chroot, options, args):
             chroot.clean()
         raise
 
+def rootcheck():
+    "verify mock was started correctly (either by sudo or consolehelper)"
+    # if we're root due to sudo or consolehelper, we're ok
+    # if not raise an exception and bail
+    if os.getuid() == 0 and not (os.environ.get("SUDO_UID") or os.environ.get("USERHELPER_UID")):
+        raise RuntimeError, "mock will not run from the root account (needs an unprivileged uid so it can drop privs)"
+
+def groupcheck(unprivGid, tgtGid):
+    "verify that the user running mock is part of the correct group"
+    # verify that we're in the correct group (so all our uid/gid manipulations work)
+    inmockgrp = False
+    members = []
+    for gid in os.getgroups() + [unprivGid]:
+        name = grp.getgrgid(gid).gr_name
+        if gid == tgtGid:
+            inmockgrp = True
+            break
+        members.append(name)
+    if not inmockgrp:
+        name = grp.getgrgid(tgtGid).gr_name
+        raise RuntimeError("Must be member of '%s' group to run mock! (%s)" %
+                           (name, ", ".join(members)))
+
 def main(ret):
     "Main executable entry point."
-    # drop unprivleged to parse args, etc.
-    #   uidManager saves current real uid/gid which are unpriviledged (callers)
+
+    # initial sanity check for correct invocation method
+    rootcheck()
+
+    # drop unprivileged to parse args, etc.
+    #   uidManager saves current real uid/gid which are unprivileged (callers)
     #   due to suid helper, our current effective uid is 0
     #   also supports being run by sudo
     #
@@ -435,30 +381,36 @@ def main(ret):
     unprivUid = os.getuid()
     unprivGid = os.getgid()
 
+    mockgid = grp.getgrnam('mock').gr_gid
+
     # sudo
     if os.environ.get("SUDO_UID") is not None:
         unprivUid = int(os.environ['SUDO_UID'])
         username = os.environ.get("SUDO_USER")
-        groups = [ g[2] for g in grp.getgrall() if username in g[3]]
-        os.setgroups(groups)
+        os.setgroups((mockgid,))
         unprivGid = int(os.environ['SUDO_GID'])
 
     # consolehelper
     if os.environ.get("USERHELPER_UID") is not None:
         unprivUid = int(os.environ['USERHELPER_UID'])
         username = pwd.getpwuid(unprivUid)[0]
-        groups = [ g[2] for g in grp.getgrall() if username in g[3]]
-        os.setgroups(groups)
+        os.setgroups((mockgid,))
         unprivGid = pwd.getpwuid(unprivUid)[3]
 
-    uidManager = mock.uid.uidManager(unprivUid, unprivGid)
-    uidManager._becomeUser(unprivUid, unprivGid)
+    uidManager = mockbuild.uid.uidManager(unprivUid, unprivGid)
+    # go unpriv only when root to make --help etc work for non-mock users
+    if os.geteuid() == 0:
+        uidManager._becomeUser(unprivUid, unprivGid)
 
     # defaults
-    config_opts = {}
-    setup_default_config_opts(config_opts, unprivUid)
+    config_opts = mockbuild.util.setup_default_config_opts(unprivUid, __VERSION__, PKGPYTHONDIR)
+
     (options, args) = command_parse(config_opts)
 
+    # allow a different mock group to be specified
+    if config_opts['chrootgid'] != mockgid:
+        os.setgroups((mockgid, config_opts['chrootgid']))
+
     if options.printrootpath:
         options.verbose = 0
 
@@ -480,6 +432,17 @@ def main(ret):
             if options.chroot == "default": log.error("  Did you forget to specify the chroot to use with '-r'?")
             sys.exit(1)
 
+    # verify that our unprivileged uid is in the mock group
+    groupcheck(unprivGid, config_opts['chrootgid'])
+
+    # Read user specific config file
+    cfg = os.path.join(os.path.expanduser('~' + pwd.getpwuid(os.getuid())[0]), '.mock/user.cfg')
+    if os.path.exists(cfg):
+        config_opts['config_paths'].append(cfg)
+        uidManager.dropPrivsTemp()
+        execfile(cfg)
+        uidManager.restorePrivs()
+
     # configure logging
     config_opts['chroot_name'] = options.chroot
     log_ini = os.path.join(config_path, config_opts["log_config_file"])
@@ -504,22 +467,30 @@ def main(ret):
         log.error("Log config file (%s) missing required section: %s" % (log_ini, exc))
         sys.exit(50)
 
+    # set logging verbosity
     if options.verbose == 0:
         log.handlers[0].setLevel(logging.WARNING)
-        logging.getLogger("mock.Root.state").handlers[0].setLevel(logging.WARNING)
+        tmplog = logging.getLogger("mockbuild.Root.state")
+        if tmplog.handlers:
+            tmplog.handlers[0].setLevel(logging.WARNING)
     elif options.verbose == 1:
         log.handlers[0].setLevel(logging.INFO)
     elif options.verbose == 2:
         log.handlers[0].setLevel(logging.DEBUG)
-        logging.getLogger("mock.Root.build").propagate = 1
-        logging.getLogger("mock").propagate = 1
+        logging.getLogger("mockbuild.Root.build").propagate = 1
+        logging.getLogger("mockbuild").propagate = 1
 
+    # enable tracing if requested
     logging.getLogger("trace").propagate=0
     if options.trace:
         logging.getLogger("trace").propagate=1
 
     # cmdline options override config options
-    set_config_opts_per_cmdline(config_opts, options, args)
+    #set_config_opts_per_cmdline(config_opts, options, args)
+    mockbuild.util.set_config_opts_per_cmdline(config_opts, options, args)
+
+    # verify that we're not trying to build an arch that we can't
+    check_arch_combination(config_opts['rpmbuild_arch'], config_opts)
 
     # default /etc/hosts contents
     if not config_opts['use_host_resolv'] and not config_opts['files'].has_key('etc/hosts'):
@@ -527,12 +498,25 @@ def main(ret):
 127.0.0.1 localhost localhost.localdomain
 ::1       localhost localhost.localdomain localhost6 localhost6.localdomain6
 '''
+
+    # Fetch and prepare sources from SCM
+    if config_opts['scm']:
+        scmWorker = mockbuild.scm.scmWorker(log, config_opts['scm_opts'])
+        scmWorker.get_sources()
+        (options.sources, options.spec) = scmWorker.prepare_sources()
+
+    # security cleanup (don't need/want this in the chroot)
+    if os.environ.has_key('SSH_AUTH_SOCK'):
+        del os.environ['SSH_AUTH_SOCK']
+
     # elevate privs
     uidManager._becomeUser(0, 0)
 
     # do whatever we're here to do
     log.info("mock.py version %s starting..." % __VERSION__)
-    chroot = mock.backend.Root(config_opts, uidManager)
+    chroot = mockbuild.backend.Root(config_opts, uidManager)
+
+    chroot.start("run")
 
     if options.printrootpath:
         print chroot.makeChrootPath('')
@@ -549,14 +533,21 @@ def main(ret):
     os.environ["HOME"] = chroot.homedir
 
     # New namespace starting from here
+    base_unshare_flags = mockbuild.util.CLONE_NEWNS
+    extended_unshare_flags = base_unshare_flags|mockbuild.util.CLONE_NEWIPC|mockbuild.util.CLONE_NEWUTS
     try:
-        mock.util.unshare(mock.util.CLONE_NEWNS)
-    except:
-        log.info("Namespace unshare failed.")
+        mockbuild.util.unshare(extended_unshare_flags)
+    except mockbuild.exception.UnshareFailed, e:
+        log.debug("unshare(%d) failed, falling back to unshare(%d)" % (extended_unshare_flags, base_unshare_flags))
+        try:
+            mockbuild.util.unshare(base_unshare_flags)
+        except mockbuild.exception.UnshareFailed, e:
+            log.error("Namespace unshare failed.")
+            sys.exit(e.resultcode)
 
     # set personality (ie. setarch)
     if config_opts['internal_setarch']:
-        mock.util.condPersonality(config_opts['target_arch'])
+        mockbuild.util.condPersonality(config_opts['target_arch'])
 
     if options.mode == 'init':
         if config_opts['clean']:
@@ -564,54 +555,36 @@ def main(ret):
         chroot.init()
 
     elif options.mode == 'clean':
-        chroot.clean()
+        if len(options.scrub) == 0:
+            chroot.clean()
+        else:
+            chroot.scrub(options.scrub)
 
     elif options.mode == 'shell':
-        chroot.tryLockBuildRoot()
         if not os.path.exists(chroot.makeChrootPath()):
-            raise RuntimeError, "chroot %s not initialized!" % chroot.makeChrootPath()
-        try:
-            chroot._setupDev()
-            chroot._mountall()
+            raise mockbuild.exception.ChrootNotInitialized, \
+                "chroot %s not initialized!" % chroot.makeChrootPath()
+        if len(args):
             cmd = ' '.join(args)
-            if options.unpriv:
-                arg = '--userspec=%s:%s' % (chroot.chrootuid, chroot.chrootgid)
-            else:
-                arg = ''
-            status = os.system("PS1='mock-chroot> ' /usr/sbin/chroot %s %s %s" % (arg, chroot.makeChrootPath(), cmd))
-            ret['exitStatus'] = os.WEXITSTATUS(status)
-
-        finally:
-            chroot._umountall()
+        else:
+            cmd = None
+        sys.exit(chroot.shell(options, cmd))
 
     elif options.mode == 'chroot':
-        shell=False
+        if not os.path.exists(chroot.makeChrootPath()):
+            raise mockbuild.exception.ChrootNotInitialized, \
+                "chroot %s not initialized!" % chroot.makeChrootPath()
         if len(args) == 0:
-            log.critical("You must specify a command to run")
+            log.critical("You must specify a command to run with --chroot")
             sys.exit(50)
-        elif len(args) == 1:
-            args = args[0]
-            shell=True
-
-        log.info("Running in chroot: %s" % args)
-        chroot.tryLockBuildRoot()
-        chroot._resetLogging()
-        try:
-            chroot._mountall()
-            if options.unpriv:
-                chroot.doChroot(args, shell=shell,
-                                uid=chroot.chrootuid, gid=chroot.chrootgid, cwd=options.cwd)
-            else:
-                chroot.doChroot(args, shell=shell, cwd=options.cwd)
-        finally:
-            chroot._umountall()
+        chroot.chroot(args, options)
 
     elif options.mode == 'installdeps':
         if len(args) == 0:
-            log.critical("You must specify an SRPM file.")
+            log.critical("You must specify an SRPM file with --installdeps")
             sys.exit(50)
 
-        for hdr in mock.util.yieldSrpmHeaders(args, plainRpmOk=1):
+        for hdr in mockbuild.util.yieldSrpmHeaders(args, plainRpmOk=1):
             pass
         chroot.tryLockBuildRoot()
         try:
@@ -619,26 +592,47 @@ def main(ret):
             chroot.installSrpmDeps(*args)
         finally:
             chroot._umountall()
+        chroot.unlockBuildRoot()
 
     elif options.mode == 'install':
         if len(args) == 0:
             log.critical("You must specify a package list to install.")
             sys.exit(50)
 
+        chroot._resetLogging()
         chroot.tryLockBuildRoot()
         chroot.yumInstall(*args)
+        chroot.unlockBuildRoot()
 
     elif options.mode == 'update':
+        chroot._resetLogging()
+        chroot.tryLockBuildRoot()
         chroot.yumUpdate()
+        chroot.unlockBuildRoot()
+
+    elif options.mode == 'remove':
+        if len(args) == 0:
+            log.critical("You must specify a package list to remove.")
+            sys.exit(50)
+
+        chroot._resetLogging()
+        chroot.tryLockBuildRoot()
+        chroot.yumRemove(*args)
+        chroot.unlockBuildRoot()
 
     elif options.mode == 'rebuild':
+        if config_opts['scm']:
+            srpm = do_buildsrpm(config_opts, chroot, options, args)
+            if srpm:
+                args.append(srpm)
+            scmWorker.clean()
         do_rebuild(config_opts, chroot, args)
 
     elif options.mode == 'buildsrpm':
         do_buildsrpm(config_opts, chroot, options, args)
 
     elif options.mode == 'orphanskill':
-        mock.util.orphansKill(chroot.makeChrootPath())
+        mockbuild.util.orphansKill(chroot.makeChrootPath())
     elif options.mode == 'copyin':
         chroot.tryLockBuildRoot()
         chroot._resetLogging()
@@ -653,11 +647,13 @@ def main(ret):
         args = args[:-1]
         import shutil
         for src in args:
-            log.debug("copying %s to %s" % (src, dest))
+            log.info("copying %s to %s" % (src, dest))
             if os.path.isdir(src):
                 shutil.copytree(src, dest)
             else:
                 shutil.copy(src, dest)
+        chroot.unlockBuildRoot()
+
     elif options.mode == 'copyout':
         chroot.tryLockBuildRoot()
         chroot._resetLogging()
@@ -673,11 +669,16 @@ def main(ret):
         import shutil
         for f in args:
             src = chroot.makeChrootPath(f)
-            log.debug("copying %s to %s" % (src, dest))
+            log.info("copying %s to %s" % (src, dest))
             if os.path.isdir(src):
                 shutil.copytree(src, dest)
             else:
                 shutil.copy(src, dest)
+        chroot.unlockBuildRoot()
+
+    chroot._nuke_rpm_db()
+    chroot.finish("run")
+    chroot.alldone()
 
 if __name__ == '__main__':
     # fix for python 2.4 logging module bug:
@@ -711,17 +712,17 @@ if __name__ == '__main__':
         exitStatus = 7
         log.error("Exiting on user interrupt, <CTRL>-C")
 
-    except (mock.exception.ResultDirNotAccessible,), exc:
+    except (mockbuild.exception.ResultDirNotAccessible,), exc:
         exitStatus = exc.resultcode
         log.error(str(exc))
         killOrphans = 0
 
-    except (mock.exception.BadCmdline, mock.exception.BuildRootLocked), exc:
+    except (mockbuild.exception.BadCmdline, mockbuild.exception.BuildRootLocked), exc:
         exitStatus = exc.resultcode
         log.error(str(exc))
         killOrphans = 0
 
-    except (mock.exception.Error), exc:
+    except (mockbuild.exception.Error), exc:
         exitStatus = exc.resultcode
         log.error(str(exc))
 
@@ -730,10 +731,7 @@ if __name__ == '__main__':
         log.exception(exc)
 
     if killOrphans and retParams:
-        mock.util.orphansKill(retParams["chroot"].makeChrootPath())
+        mockbuild.util.orphansKill(retParams["chroot"].makeChrootPath())
 
     logging.shutdown()
     sys.exit(exitStatus)
-
-
-
